/* $Id: mcl_tx.cpp,v 1.12 2004/11/29 17:07:34 chneuman Exp $ */
/*
 *  Copyright (c) 1999-2004 INRIA - Universite Paris 6 - All rights reserved
 *  (main author: Vincent Roca - vincent.roca@inrialpes.fr)
 *
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License
 *  as published by the Free Software Foundation; either version 2
 *  of the License, or (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
 *  USA.
 */

#include "mcl_includes.h"


/* private functions */
static int	mcl_sendit	(mcl_cb *mclcb, txlay_t *tl, int du_nb, int du_nb_high);
#ifdef LCT_SCHED4
static int	mcl_sda_sendit	(mcl_cb *mclcb, txlay_t *tl, int size);
#endif


/*
 * for packet traces
 * store msg in a buf and print everything atomically
 */
#define TRC_BUF_SIZE	1024
static char	trc_buf[TRC_BUF_SIZE];
static UINT32	trc_offset = 0;
#define ADD_TO_STR_BUF(m)	{ if (trc_offset + strlen(m) < TRC_BUF_SIZE) { \
					strcpy(&trc_buf[trc_offset], m);     \
					trc_offset += strlen(m);	     \
				} }


/*
 *
 */
void
mcl_try_to_send (mcl_cb	*mclcb)
{
        int		i;
	int		du_nb = 0;		/* number of DUs to send */
	int		du_nb_high = 0;		/* number of DUs to send */
	txlay_t		*tl;
	bool		something_sent;	/* has any DU been sent on base layer?*/

	TRACELVL(5, (mcl_stdout, "-> mcl_try_to_send: mclcb=x%x\n", (int)mclcb))
	/*
	 * see what to send next for each tx layer.
	 */
	if (mclcb->is_a_sender() && !mclcb->fsm.is_closed(mclcb)) {
		something_sent= false;	/* reset first */
		mclcb->more_to_tx = 0;	/* reset first */
		for (i = 0, tl = mclcb->txlay_tab; i < mclcb->nb_layers;
		     i++, tl++) {
			if (tl->tx_high_timer > 0) {
				tl->tx_high_timer--;
			} else {
				/* number of ticks per second (in order to
				 * send important data every second) */
				tl->tx_high_timer = (int)IMPORTANT_DATA_FREQUENCY / MCL_TIMER_PERIOD;
			}
			/*
			 * how many DUs (of high or normal priority) should
			 * be sent?
			 */
			if (tl->tx_high_timer == 0) {
				du_nb_high = min(tl->du_per_tick,
						 tl->tx_tab_high->tot_rem);
			} else {
			 	du_nb_high = 0;
			}
			du_nb = min(tl->du_per_tick - du_nb_high,
				    tl->tx_tab->tot_rem);
			if (du_nb_high + du_nb > 0) {
				/*
				 * something to send!
				 */
				if (mclcb->first_tx_for_mclcb) {
					mclcb->first_tx_for_mclcb = 0;

					/* reset SP at first pkt tx */
#ifdef RLC
					if (mclcb->congestion_control == RLC_CC) rlc_reset_tx_sp(mclcb);
#endif
				}
				mcl_sendit(mclcb, tl, du_nb, du_nb_high);
				if (i == 0) {
					/* something sent on base layer */
					something_sent = true;
				}
			}
			if (tl->tx_tab->tot_rem == 0) {
				/*
				 * get ready for a new tx cycle
				 * on that layer for tx_tab, if needed.
				 */
				tl->tx_tab->new_tx_cycle(mclcb);
			}
			
			if (tl->tx_tab_high->tot_rem == 0) {
				/*
				 * get ready for a new tx cycle
				 * on that layer for tx_tab_high, if needed.
				 */
				tl->tx_tab_high->new_tx_cycle(mclcb);
			}
		}
		/*
		 * print all the packet traces atomically
		 */
		if (trc_offset > 0) {
			ADD_TO_STR_BUF("\n");
			PRINT_OUT((mcl_stdout, trc_buf))
			trc_offset = 0;
		}
		if (something_sent == true)
			mclcb->more_to_tx = 1;
		if (something_sent == false && mclcb->fsm.no_new_adu(mclcb)) {
			/*
			 * no transmission on base layer!
			 * stop everything and send a close message...
			 */
			mclcb->fsm.update_tx_state(mclcb, TEVENT_ALL_DU_SENT);
			SendCLOSE(mclcb);
			mclcb->fsm.update_tx_state(mclcb, TEVENT_CLOSE_SENT);
		}
	}
	TRACELVL(5, (mcl_stdout, "<- mcl_try_to_send:\n"))
}


/*
 * Send this number of DUs for this layer
 */
static int
mcl_sendit (mcl_cb	*mclcb,
	    txlay_t	*tl,
	    int		du_nb,
	    int		du_nb_high)
{
	du_t		*du = NULL;
	int		rem;
	char		s[64];	/* for string */
	
	TRACELVL(5, (mcl_stdout,
		"-> mcl_sendit: lay %d: %d DUs, %d high prio DUs\n",
		tl->layer, du_nb, du_nb_high))
	ASSERT(du_nb > 0 || du_nb_high > 0);
	ASSERT(tl && (tl->tx_tab->tot_rem >= du_nb));
	ASSERT((tl->tx_tab_high->tot_rem >= du_nb_high));

	if (mclcb->get_verbosity() == 1)
		ADD_TO_STR_BUF("\t\t")

	/* high importance data */
	for (rem = du_nb_high; rem > 0; rem--) {
		du = tl->tx_tab_high->get_next_du (mclcb);
		ASSERT(du);
		if (mclcb->get_verbosity() == 1) {
			/* compact, imprecise but easily readable trace */
#ifdef FEC
			if (du->is_fec)
				sprintf(s, "((%d/%d/%d)) ", du->block->adu->seq,
					du->block->seq, du->seq);
			else
#endif /* FEC */
				sprintf(s, "[%d/%d/%d] ", du->block->adu->seq,
					du->block->seq, du->seq);
			ADD_TO_STR_BUF(s)
		}
		/* then send data*/
		mcl_send_pkt(mclcb, tl->layer, du, du->block->adu);
	}

	/* normal importance data */	
	for (rem = du_nb; rem > 0; rem--) {
		du = tl->tx_tab->get_next_du (mclcb);
		ASSERT(du);
		if (mclcb->get_verbosity() == 1) {
			/* compact, imprecise but easily readable trace */
#ifdef FEC
			if (du->is_fec)
				sprintf(s, "((%d/%d/%d)) ", du->block->adu->seq, du->block->seq, du->seq);
			else
#endif /* FEC */
				sprintf(s, "[%d/%d/%d] ", du->block->adu->seq, du->block->seq, du->seq);
			ADD_TO_STR_BUF(s)
		}
		/* then send data*/
		mcl_send_pkt(mclcb, tl->layer, du, du->block->adu);
	}
	TRACELVL(5, (mcl_stdout, "<- mcl_sendit:\n"))
	return 0;
}


/*
 * scan all the remaining du transmissions and free
 * buffers for ADUs completely sent
 */
void
mcl_tx_cleanup (mcl_cb	*mclcb)
{
	adu_t		*list;		/* head of adu list */
	adu_t		*adu;
	txlay_t		*tl;
	int		i;

	TRACELVL(5, (mcl_stdout, "-> mcl_tx_cleanup:\n"))
	ASSERT(mclcb->delivery_mode == DEL_MODE_PUSH);
	if ((list = mclcb->txlay_tab[0].adu_head) != NULL) {
		/*
		 * reset the in_txtab flag for each ADU
		 */
		adu = list;
		do {
#ifdef DEBUG
			if (adu->blocking_struct.block_nb == 0) {
				/* already free'ed, just to check */
				ASSERT((adu->block_head == NULL &&
					adu->data == NULL))
			}
#endif /* DEBUG */
			adu->in_txtab = 0;	/* by default not in txtab */
			adu = adu->next;
		} while (adu != list);
		/*
		 * search in txtab all the layers which ADUs have pending DUs
		 */
		for (i = 0, tl = mclcb->txlay_tab; i < mclcb->nb_layers;
		     i++, tl++) {
			 tl->tx_tab->mark_adus_in_txtab(mclcb);
			 tl->tx_tab_high->mark_adus_in_txtab(mclcb);
		}
		/*
		 * and free ADUs with no pending DU
		 */
		adu = list;
		do {
			if (adu->blocking_struct.block_nb > 0 && adu->in_txtab == 0) {
				/*
				 * everything has been sent; free this ADU
				 * nb: in fact adu buffer is not free'ed!
				 */
				mcl_tx_free_this_adu(mclcb, adu);
				ASSERT((adu->blocking_struct.block_nb == 0 &&
					adu->block_head == NULL &&
					adu->data == NULL))
			}
			adu = adu->next;
		} while (adu != list);
	}
	TRACELVL(5, (mcl_stdout, "<- mcl_tx_cleanup: ok\n"))
}

